from pydantic import BaseModel, Field
from dataclasses import dataclass

from typing import Type
import httpx

from argos.schemas import Task


class Status:
    ON_CHECK = "on-check"
    SUCCESS = "success"
    FAILURE = "failure"


# XXX We could name this Result, but is it could overlap with schemas.Result.
# Need to better define the naming around this.
# Status can be "Success" / "Failure" / "Error" or "On Check"
@dataclass
class Response:
    status: str
    context: dict

    @classmethod
    def new(cls, status, **kwargs):
        if type(status) == bool:
            status = Status.SUCCESS if status else Status.FAILURE

        return cls(status=status, context=kwargs)


class BaseExpectedValue(BaseModel):
    expected: str

    def get_converted(self):
        return self.expected


class ExpectedIntValue(BaseExpectedValue):
    def get_converted(self):
        return int(self.expected)


class ExpectedStringValue(BaseExpectedValue):
    pass


class CheckNotFound(Exception):
    pass


class InvalidResponse(Exception):
    def __str__(self):
        return "The provided response is missing a 'status' key."


class BaseCheck:
    config: str
    expected_cls: Type[BaseExpectedValue] = None

    _registry = []

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls._registry.append(cls)

    @classmethod
    def get_registered_checks(cls):
        return {c.config: c for c in cls._registry}

    @classmethod
    def get_registered_check(cls, name):
        check = cls.get_registered_checks().get(name)
        if not check:
            raise CheckNotFound(name)
        return check

    def __init__(self, http_client: httpx.AsyncClient, task: Task):
        self.http_client = http_client
        self.task = task

    @property
    def expected(self):
        return self.expected_cls(expected=self.task.expected).get_converted()

    def response(self, **kwargs):
        if "status" not in kwargs:
            raise InvalidResponse(kwargs)
        status = kwargs.pop("status")
        return Response.new(status, **kwargs)


def get_check_by_name(name):
    return BaseCheck.get_registered_check(name)
